SystemOrganization addCategory: #LostInMorvanBlog!
SystemOrganization addCategory: 'LostInMorvanBlog-Seaside'!
SystemOrganization addCategory: 'LostInMorvanBlog-Teapot'!
SystemOrganization addCategory: 'LostInMorvanBlog-Teapot-Tests'!
SystemOrganization addCategory: 'LostInMorvanBlog-Tests'!

TBSMagritteReport subclass: #TinyBlogPostsReport
	instanceVariableNames: 'blog'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LostInMorvanBlog-Seaside'!

!TinyBlogPostsReport commentStamp: 'FrancoisJarriges 5/29/2016 12:15' prior: 0!
I represent a Magritte automatic report built on a filtered collection of posts from the current blog. The blog identity is given by my parent component at creation time.
 
Internal Representation and Key Implementation Points.

    Instance Variables
	blog:		<TinyBlog>


    Implementation Points
	TinyPost instance variables have to be described with Magritte description methods and a container defined for them  (see the descriptions protocol of TinyPost class)!

!TinyBlogPostsReport class methodsFor: 'data' stamp: 'FrancoisJarriges 5/30/2016 01:46'!
addCommandsTo: aNewPostsReport
	"Here we add commands to each post in the new TinyBlogPostsReport"

	aNewPostsReport addColumn: (MACommandColumn new
		addCommandOn: aNewPostsReport selector: #viewPost: text: 'View'; yourself;
		addCommandOn: aNewPostsReport selector: #editPost: text: 'Edit'; yourself;
		addCommandOn: aNewPostsReport selector: #deletePost: text: 'Delete'; yourself).! !

!TinyBlogPostsReport class methodsFor: 'data' stamp: 'FrancoisJarriges 5/29/2016 10:36'!
filteredDescriptionFrom: anMAContainerOfTinyPostDescriptions
	"Here we return a selection of all descriptions that have been declared for a TinyPost"
	"The accessor method provides access to the MASelectorAccessor variable from the receiving MADescription"

	^ anMAContainerOfTinyPostDescriptions select: [ :eachTinyPostDescription |
		#(title category date) includes: eachTinyPostDescription accessor selector ]! !

!TinyBlogPostsReport class methodsFor: 'instance-creation' stamp: 'FrancoisJarriges 5/29/2016 12:07'!
from: aTinyBlog
	"Create a filtered Magritte report based on the posts of the designated TinyBlog"

	| newPostsReport allBlogPosts |

	allBlogPosts := aTinyBlog allBlogPosts.
	newPostsReport := self 
		rows: allBlogPosts 
		description: 
			(self filteredDescriptionFrom: allBlogPosts anyOne magritteDescription).
			
	newPostsReport blog: aTinyBlog.		
	self addCommandsTo: newPostsReport.

	^ newPostsReport
	 ! !

!TinyBlogPostsReport methodsFor: 'actions' stamp: 'FrancoisJarriges 5/30/2016 02:21'!
addPost

	| answeredTinyPost |
	
	answeredTinyPost := self call: (
		self renderPostFormFor: TinyPost new
			withSave: 'Add post'
			withCancel: 'Cancel').
	answeredTinyPost ifNotNil: [
		blog addBlogPost: answeredTinyPost.
 		self refreshReport
	]! !

!TinyBlogPostsReport methodsFor: 'accessing' stamp: 'FrancoisJarriges 5/29/2016 11:33'!
blog: anObject
	blog := anObject! !

!TinyBlogPostsReport methodsFor: 'actions' stamp: 'FrancoisJarriges 5/29/2016 17:21'!
deletePost: aTinyPost

	(self confirm: 'Do you want remove this post ?') ifTrue: [ 
		blog removeBlogPost: aTinyPost.
		self refreshReport ]
	
! !

!TinyBlogPostsReport methodsFor: 'actions' stamp: 'FrancoisJarriges 5/29/2016 17:45'!
editPost: aTinyPost

	| answeredTinyPost |
	
	answeredTinyPost := self call: (
		self renderPostFormFor: aTinyPost
			withSave: 'Save post'
			withCancel: 'Cancel').
	answeredTinyPost ifNotNil: [ blog save ]
! !

!TinyBlogPostsReport methodsFor: 'actions' stamp: 'FrancoisJarriges 5/30/2016 02:10'!
refreshReport

	self rows: blog allBlogPosts.
	self refresh
! !

!TinyBlogPostsReport methodsFor: 'rendering' stamp: 'FrancoisJarriges 5/29/2016 12:13'!
renderContentOn: html

	html tbsGlyphIcon perform: #iconPencil.
	html anchor
		callback: [ self addPost ];
		with: 'Add post'.
		super renderContentOn: html
 ! !

!TinyBlogPostsReport methodsFor: 'rendering' stamp: 'FrancoisJarriges 5/30/2016 01:33'!
renderPostFormFor: aTinyPost withSave: saveString withCancel: cancelString
	"The Magritte message asComponent build a renderable MAContainerComponent from the raw TinyBlog object."
	"After being called by an action method (addPost, editPost, viewPost) this component will answers a valid TinyBlog object or nil"
	
	| thisFormButtons |

	saveString 
		ifNil: [ thisFormButtons := { #cancel -> cancelString } ]
		ifNotNil: [ thisFormButtons := { #save -> saveString . #cancel -> cancelString } ].
	
	^ aTinyPost asComponent
		addDecoration: (TBSMagritteFormDecoration buttons: thisFormButtons);
		yourself
! !

!TinyBlogPostsReport methodsFor: 'actions' stamp: 'FrancoisJarriges 5/29/2016 17:47'!
viewPost: aTinyPost

	self call: (
		self renderPostFormFor: aTinyPost
			withSave: nil
			withCancel: 'Back')! !

TestCase subclass: #TinyBlogTest
	instanceVariableNames: 'blog numberOfPosts additionalEmptyPost category1 category2 numberOfPostsFromCategory1 numberOfPostsFromCategory2 numberOfPostsUnclassified numberOfCategories categoriesClass numberOfCategoriesFromVisiblePosts numberOfVisiblePosts numberOfVisiblePostsUnclassified numberOfNotVisiblePostsUnclassified numberOfCategoriesFromNotVisiblePosts'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LostInMorvanBlog-Tests'!

!TinyBlogTest methodsFor: 'running' stamp: 'FrancoisJarriges 5/22/2016 13:45'!
setUp

	blog := TinyBlog createDemoPosts current.
	
	numberOfPosts := 5.
	numberOfVisiblePosts := 5.
	numberOfVisiblePostsUnclassified := 1.
	numberOfNotVisiblePostsUnclassified := 0.
	
	category1 := 'TinyBlog'.
	category2 := 'Pharo'.
	
	categoriesClass := TinyPost unclassifiedTag class.
	
	numberOfPostsFromCategory1 := 1.
	numberOfPostsFromCategory2 := 3.
	numberOfPostsUnclassified  := 1.
	
	numberOfCategories := 3.
	numberOfCategoriesFromVisiblePosts := 3.
	numberOfCategoriesFromNotVisiblePosts := 0.

	
	additionalEmptyPost := TinyPost new! !

!TinyBlogTest methodsFor: 'running' stamp: 'FrancoisJarriges 5/22/2016 13:44'!
tearDown
	"reset the current blog at the end of the tests"

	TinyBlog reset! !

!TinyBlogTest methodsFor: 'tests-actions' stamp: 'FrancoisJarriges 5/11/2016 11:40'!
testAddBlogPost

	blog addBlogPost: additionalEmptyPost.
	self assert: blog size equals: numberOfPosts + 1! !

!TinyBlogTest methodsFor: 'tests-actions' stamp: 'FrancoisJarriges 5/27/2016 11:30'!
testAllBlogPosts

	| allPosts |
	
	allPosts := blog allBlogPosts.
	self assert: allPosts size equals: numberOfPosts.
	allPosts do: [ :eachPost | 
		self assert: eachPost class == TinyPost ]
! !

!TinyBlogTest methodsFor: 'tests-actions' stamp: 'FrancoisJarriges 5/27/2016 11:48'!
testAllBlogPostsFromCategory
	| postsFromCategory1 postsFromCategory2 postsUnclassified |
	postsFromCategory1 := blog allBlogPostsFromCategory: category1.
	self
		assert: postsFromCategory1 size
		equals: numberOfPostsFromCategory1.
	postsFromCategory1
		do: [ :eachPost | self assert: eachPost class == TinyPost ].
	postsFromCategory2 := blog allBlogPostsFromCategory: category2.
	self
		assert: postsFromCategory2 size
		equals: numberOfPostsFromCategory2.
	postsFromCategory2
		do: [ :eachPost | self assert: eachPost class == TinyPost ].
	postsUnclassified := blog
		allBlogPostsFromCategory: TinyPost unclassifiedTag.
	self assert: postsUnclassified size equals: numberOfPostsUnclassified.
	postsUnclassified
		do: [ :eachPost | self assert: eachPost class == TinyPost ]! !

!TinyBlogTest methodsFor: 'tests-actions' stamp: 'FrancoisJarriges 5/27/2016 11:48'!
testAllCategories
	| allCategories |
	allCategories := blog allCategories.
	self assert: allCategories size equals: numberOfCategories.
	allCategories
		do:
			[ :eachCategory | self assert: eachCategory class == categoriesClass ]! !

!TinyBlogTest methodsFor: 'tests-actions' stamp: 'FrancoisJarriges 5/27/2016 11:48'!
testAllCategoriesFromNotVisibleBlogPosts
	| categoriesFromNotVisiblePosts |
	blog addBlogPost: additionalEmptyPost.
	additionalEmptyPost category: 'only for this test'.
	categoriesFromNotVisiblePosts := blog
		allCategoriesFromNotVisibleBlogPosts.
	self
		assert: categoriesFromNotVisiblePosts size
		equals: numberOfCategoriesFromNotVisiblePosts + 1.
	categoriesFromNotVisiblePosts
		do:
			[ :eachCategory | self assert: eachCategory class == categoriesClass ]! !

!TinyBlogTest methodsFor: 'tests-actions' stamp: 'FrancoisJarriges 5/27/2016 11:48'!
testAllCategoriesFromVisibleBlogPosts
	| categoriesFromVisiblePosts |
	categoriesFromVisiblePosts := blog allCategoriesFromVisibleBlogPosts.
	self
		assert: categoriesFromVisiblePosts size
		equals: numberOfCategoriesFromVisiblePosts.
	categoriesFromVisiblePosts
		do:
			[ :eachCategory | self assert: eachCategory class == categoriesClass ]! !

!TinyBlogTest methodsFor: 'tests-actions' stamp: 'FrancoisJarriges 5/27/2016 11:48'!
testAllNotVisibleBlogPosts
	| notVisiblePosts |
	blog addBlogPost: additionalEmptyPost.
	notVisiblePosts := blog allNotVisibleBlogPosts.
	self
		assert: notVisiblePosts size
		equals: numberOfPosts - numberOfVisiblePosts + 1.
	notVisiblePosts
		do: [ :eachPost | self assert: eachPost class == TinyPost ]! !

!TinyBlogTest methodsFor: 'tests-actions' stamp: 'FrancoisJarriges 5/27/2016 11:49'!
testAllNotVisibleBlogPostsFromCategory
	| notVisiblePostsUnclassified |
	blog addBlogPost: additionalEmptyPost.
	notVisiblePostsUnclassified := blog
		allNotVisibleBlogPostsFromCategory: TinyPost unclassifiedTag.
	self
		assert: notVisiblePostsUnclassified size
		equals: numberOfNotVisiblePostsUnclassified + 1.
	notVisiblePostsUnclassified
		do: [ :eachPost | self assert: eachPost class == TinyPost ]! !

!TinyBlogTest methodsFor: 'tests-actions' stamp: 'FrancoisJarriges 5/27/2016 11:49'!
testAllVisibleBlogPosts
	| visiblePosts |
	blog addBlogPost: additionalEmptyPost.
	visiblePosts := blog allVisibleBlogPosts.
	self assert: visiblePosts size equals: numberOfVisiblePosts.
	visiblePosts
		do: [ :eachPost | self assert: eachPost class == TinyPost ]! !

!TinyBlogTest methodsFor: 'tests-actions' stamp: 'FrancoisJarriges 5/27/2016 11:49'!
testAllVisibleBlogPostsFromCategory
	| visiblePostsUnclassified |
	blog addBlogPost: additionalEmptyPost.
	visiblePostsUnclassified := blog
		allVisibleBlogPostsFromCategory: TinyPost unclassifiedTag.
	self
		assert: visiblePostsUnclassified size
		equals: numberOfVisiblePostsUnclassified.
	visiblePostsUnclassified
		do: [ :eachPost | self assert: eachPost class == TinyPost ]! !

!TinyBlogTest methodsFor: 'tests-actions' stamp: 'FrancoisJarriges 5/27/2016 11:51'!
testCategoryBeNotVisible
	| postsUnclassified visiblePostsUnclassified |
	postsUnclassified := blog
		categoryBeNotVisible: TinyPost unclassifiedTag.
	self assert: postsUnclassified size equals: numberOfPostsUnclassified.
	postsUnclassified
		do: [ :eachPost | self assert: eachPost class == TinyPost ].
	visiblePostsUnclassified := blog
		allVisibleBlogPostsFromCategory: TinyPost unclassifiedTag.
	self assert: visiblePostsUnclassified isEmpty! !

!TinyBlogTest methodsFor: 'tests-actions' stamp: 'FrancoisJarriges 5/27/2016 11:49'!
testCategoryBeVisible
	| postsUnclassified visiblePostsUnclassified |
	blog addBlogPost: additionalEmptyPost.
	postsUnclassified := blog categoryBeVisible: TinyPost unclassifiedTag.
	self
		assert: postsUnclassified size
		equals: numberOfPostsUnclassified + 1.
	postsUnclassified
		do: [ :eachPost | self assert: eachPost class == TinyPost ].
	visiblePostsUnclassified := blog
		allVisibleBlogPostsFromCategory: TinyPost unclassifiedTag.
	self
		assert: visiblePostsUnclassified size
		equals: numberOfVisiblePostsUnclassified + 1! !

!TinyBlogTest methodsFor: 'tests-actions' stamp: 'FrancoisJarriges 5/27/2016 11:49'!
testCategoryRenameFromTo
	| postsFromCategory1 postsUnclassified |
	postsUnclassified := blog
		categoryRenameFrom: TinyPost unclassifiedTag
		to: category1.
	self assert: postsUnclassified size equals: numberOfPostsUnclassified.
	postsUnclassified
		do: [ :eachPost | self assert: eachPost class == TinyPost ].
	postsFromCategory1 := blog allBlogPostsFromCategory: category1.
	self
		assert: postsFromCategory1 size
		equals: numberOfPostsFromCategory1 + numberOfPostsUnclassified! !

!TinyBlogTest methodsFor: 'tests-actions' stamp: 'FrancoisJarriges 5/11/2016 12:34'!
testRemoveAllBlogPosts

	blog removeAllBlogPosts.
	self assert: blog size equals: 0! !

!TinyBlogTest methodsFor: 'tests-actions' stamp: 'FrancoisJarriges 5/27/2016 11:49'!
testRemoveAllBlogPostsFromCategory
	| postsFromCategory2 |
	blog removeAllBlogPostsFromCategory: category2.
	postsFromCategory2 := blog allBlogPostsFromCategory: category2.
	self assert: postsFromCategory2 size equals: 0.
	self
		assert: blog allBlogPosts size
		equals: numberOfPosts - numberOfPostsFromCategory2! !

!TinyBlogTest methodsFor: 'tests-actions' stamp: 'FrancoisJarriges 5/11/2016 16:43'!
testRemoveBlogPost

	blog addBlogPost: additionalEmptyPost.
	blog removeBlogPost: additionalEmptyPost.
	self assert: blog size equals: numberOfPosts! !

!TinyBlogTest methodsFor: 'tests-accessing' stamp: 'FrancoisJarriges 5/11/2016 10:36'!
testSize
	"verify the blog size after tests setup"

	self assert: blog size equals: numberOfPosts ! !

TestCase subclass: #TinyTeapotWebAppTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LostInMorvanBlog-Teapot-Tests'!

!TinyTeapotWebAppTest commentStamp: 'FrancoisJarriges 5/22/2016 14:55' prior: 0!
I represents a TestCase for  testing the  TinyBlog  web application using the Teapot framework with a demo blog created by TinyBlog>>createDemoPosts .

The actual test is made by navigating with a web browser from the URI  http://localhost:8081/

The method TinyTeapotWebAppTest>>testTinyTeapotWebApp is used only for initializing the Teapot server with a usable demo blog.
 
As playing with the demo server is no more required the application is shut down by executing the TinyTeapotWebAppTestStop TestCase!

!TinyTeapotWebAppTest methodsFor: 'running' stamp: 'FrancoisJarriges 5/22/2016 13:45'!
setUp

	TinyBlog createDemoPosts.
	TinyTeapotWebApp start! !

!TinyTeapotWebAppTest methodsFor: 'tests-web' stamp: 'FrancoisJarriges 5/22/2016 14:42'!
testTinyTeapotWebApp

	self assert: TinyTeapotWebApp server isNotNil

	! !

TestCase subclass: #TinyTeapotWebAppTestStop
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LostInMorvanBlog-Teapot-Tests'!

!TinyTeapotWebAppTestStop commentStamp: 'FrancoisJarriges 5/22/2016 14:56' prior: 0!
I represents a TestCase for  testing the  TinyBlog  web application shutdown

(see TinyTeapotWebbAppTest for explanations)!

!TinyTeapotWebAppTestStop methodsFor: 'running' stamp: 'FrancoisJarriges 5/22/2016 14:53'!
testTinyTeapotWebAppStop

	TinyTeapotWebApp stop.

	self assert: TinyTeapotWebApp server isNil
! !

WAComponent subclass: #TinyBlogAuthenticationComponent
	instanceVariableNames: 'password account'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LostInMorvanBlog-Seaside'!

!TinyBlogAuthenticationComponent commentStamp: 'FrancoisJarriges 5/28/2016 16:12' prior: 0!
I represent a small Seaside component that opens an authentication dialog box before calling the TinyBlogAdministrationComponent.

 
Internal Representation and Key Implementation Points.

    Instance Variables
	account:		<ByteString>
	component:		<WAComponent>
	password:		<ByteString>


    Implementation Points!

!TinyBlogAuthenticationComponent methodsFor: 'rendering' stamp: 'FrancoisJarriges 5/30/2016 01:03'!
renderContentOn: html

	html tbsModal id: 'TinyBlogAuthDialog'; with: [
		html tbsModalDialog: [
			html tbsModalContent: [
				self renderHeaderOn: html.
				html tbsModalBody: [
					html form: [
						self renderInputAccountOn: html.
						html break.
						self renderInputPasswordOn: html.
						html break.
						html break.
						self renderSubmitFooterOn: html ] ] ] ] ]
! !

!TinyBlogAuthenticationComponent methodsFor: 'rendering' stamp: 'FrancoisJarriges 5/30/2016 01:02'!
renderHeaderOn: html

	html tbsModalHeader: [
		html tbsModalCloseIcon.
		html tbsModalTitle level: 4; with: 'Authentication' 
	]! !

!TinyBlogAuthenticationComponent methodsFor: 'rendering' stamp: 'FrancoisJarriges 5/28/2016 16:57'!
renderInputAccountOn: html

	html text: 'Account:'.
	html break.
	html textInput
		callback: [ :value | account := value ];
		value: account
! !

!TinyBlogAuthenticationComponent methodsFor: 'rendering' stamp: 'FrancoisJarriges 5/28/2016 16:59'!
renderInputPasswordOn: html

	html text: 'Password:'.
	html break.
	html passwordInput
		callback: [ :value | password := value ];
		value: password
! !

!TinyBlogAuthenticationComponent methodsFor: 'rendering' stamp: 'FrancoisJarriges 5/28/2016 17:07'!
renderSubmitFooterOn: html

	html tbsModalFooter: [
		html tbsSubmitButton value: 'Cancel'.
		html tbsSubmitButton
			bePrimary;
			callback: [ self validate ];
			value: 'SignIn' ]! !

!TinyBlogAuthenticationComponent methodsFor: 'actions' stamp: 'FrancoisJarriges 5/29/2016 20:03'!
validate
	"This a fake method to validate user authentication."
	"It has to be implemented with an access to a backend file."

	
	(account = 'admin' and: password = 'password' ) 
		ifTrue: [ 
			self session logged: true.
			self call: TinyBlogAdministrationComponent new ]! !

WAComponent subclass: #TinyBlogCategoriesComponent
	instanceVariableNames: 'categories selectedCategory'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LostInMorvanBlog-Seaside'!

!TinyBlogCategoriesComponent commentStamp: 'FrancoisJarriges 5/29/2016 08:48' prior: 0!
I represent a component used to render a  list of  categories of posts in this tiny blog application. I get the list of categories I am rendering from my parent component at creation time. I provide an accessor to the selected category for using by  other component which  need it.


Internal Representation and Key Implementation Points.

    Instance Variables
	categories:		<OrderedCollection>
	selectedCategory		<ByteString>


    Implementation Points

!

!TinyBlogCategoriesComponent class methodsFor: 'instance-creation' stamp: 'FrancoisJarriges 5/29/2016 08:36'!
categories: anOrderedCollection 

	^ self new categories: anOrderedCollection! !

!TinyBlogCategoriesComponent methodsFor: 'accessing' stamp: 'FrancoisJarriges 5/29/2016 20:30'!
categories: anOrderedCollection
	categories := anOrderedCollection! !

!TinyBlogCategoriesComponent methodsFor: 'rendering' stamp: 'FrancoisJarriges 5/29/2016 08:43'!
renderAllCategoryLinkOn: html
	"This method uses the Bootstrap List group rendering for the 'All categories ' linked item."
	"The selected category with a nil value is equivalent to the selection of the all the categories."

	html tbsLinkifyListGroupItem
		class: 'active' if: selectedCategory isNil;
		callback: [ selectedCategory := nil ];
		with: 'All'! !

!TinyBlogCategoriesComponent methodsFor: 'rendering' stamp: 'FrancoisJarriges 5/29/2016 08:43'!
renderCategoryLinkOn: html with: aCategoryString
	"This method uses the Bootstrap List group rendering for each category linked item."
	
	html tbsLinkifyListGroupItem
		class: 'active' if: aCategoryString = selectedCategory;
		callback: [ selectedCategory := aCategoryString ];
		with: aCategoryString! !

!TinyBlogCategoriesComponent methodsFor: 'rendering' stamp: 'FrancoisJarriges 5/27/2016 16:31'!
renderCategoryMenuTitleOn: html

	html tbsListGroupItem class:'categories-menu-title'; with: 'Categories' 
! !

!TinyBlogCategoriesComponent methodsFor: 'rendering' stamp: 'FrancoisJarriges 5/27/2016 20:25'!
renderContentOn: html
	"This method uses the List group Bootstrap rendering for the list of categories."
	
	html tbsListGroup: [
		self renderCategoryMenuTitleOn: html.
		self renderAllCategoryLinkOn: html.
		categories sorted do: [ :eachCategory |
			self renderCategoryLinkOn: html with: eachCategory ] ]
! !

!TinyBlogCategoriesComponent methodsFor: 'accessing' stamp: 'FrancoisJarriges 5/29/2016 08:23'!
selectedCategory
	
	^ selectedCategory! !

WAComponent subclass: #TinyBlogHeaderComponent
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LostInMorvanBlog-Seaside'!

!TinyBlogHeaderComponent commentStamp: 'FrancoisJarriges 5/27/2016 10:38' prior: 0!
I represent the common header rendering  component for the TinyBlog application.
 
Internal Representation and Key Implementation Points.


    Implementation Points!

!TinyBlogHeaderComponent methodsFor: 'rendering' stamp: 'FrancoisJarriges 5/30/2016 00:26'!
renderContentOn: html
	
   html div 
		class: 'blog-header';
		with: [ html text: TinyBlog current title ].
	html break
! !

WAComponent subclass: #TinyBlogRootComponent
	instanceVariableNames: 'main'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LostInMorvanBlog-Seaside'!

!TinyBlogRootComponent commentStamp: 'FrancoisJarriges 5/28/2016 15:54' prior: 0!
I represent the root component of the TinyBlog application. I always contains a main TinyBlogScreenComponent subclass instance as child..
 
Internal Representation and Key Implementation Points.

    Instance Variables
	main: 		<TinyBlogScreenComponent >


    Implementation Points!

!TinyBlogRootComponent class methodsFor: 'class initialization' stamp: 'FrancoisJarriges 5/26/2016 13:48'!
canBeRoot
	"Indicate to Seaside that this class is the root component."
	"Thus it will be instancied as soon as a user connects to the  application."
	
	^ true! !

!TinyBlogRootComponent class methodsFor: 'class initialization' stamp: 'FrancoisJarriges 5/29/2016 20:26'!
initialize
	"Declare the TinyBlog application to the Seaside server."
	"Specify the TinyBlogSession object as Seaside current session object"
	"Integrate Bootstrap framework dependancies"
	"(css and js will be stored in the application)."
	
	| application |

	application := WAAdmin register: self asApplicationAt: TinyBlog current title.
	application preferenceAt: #sessionClass put: TinyBlogSession.
	application
		addLibrary: JQDeploymentLibrary;
		addLibrary: JQUiDeploymentLibrary;
		addLibrary: TBSDeploymentLibrary! !

TinyBlogRootComponent initialize!

!TinyBlogRootComponent methodsFor: 'hooks' stamp: 'FrancoisJarriges 5/26/2016 14:22'!
children
	"Seaside requires that each component knows and declares all its visible child components using the method WAComponent>>children. This allows Seaside to know in advance what components will be visible when building the HTML and configure and trigger some event on these components before the actual rendering happens."
	"Here we declare that the main TinyBlogScreenComponent instance variable is a child component of the root component"
	
   ^ { main }! !

!TinyBlogRootComponent methodsFor: 'initialization' stamp: 'FrancoisJarriges 5/26/2016 15:15'!
initialize
	"Create a default main TinyBlogScreenComponent sub-component for the root component."

	super initialize.
	main := TinyPostsListComponent new! !

!TinyBlogRootComponent methodsFor: 'rendering' stamp: 'FrancoisJarriges 5/28/2016 14:17'!
renderContentOn: html
	"may be this method is not necessary at the root level (because it works without it)"
	
   html render: main! !

!TinyBlogRootComponent methodsFor: 'hooks' stamp: 'FrancoisJarriges 5/30/2016 00:52'!
style

	^ '
		body { 
			font-family: Arial, Helvetica, sans-serif;
		}
		.categories-menu-title, .blog-header, .sign-in {
			font-family: "Arial Black", Gadget, sans-serif;
		}
		.categories-menu-title { 
			font-size-adjust: 0.58;
			color: black;
			background-color: orange;
			border-color: orange;
		}
		.blog-header { 
			color: orange;
			background-color: gray;
			text-shadow: 2px 2px black;
			font-size: xx-large;
			padding-left: 36px;
			line-height: 220%;
			letter-spacing: 2px;
			text-transform: capitalize;
			height: 85px;
		}
		.sign-in { 
			text-align: right;
			font-size: medium;
			padding-left: 5px;
			padding-right: 15px;
		}
		.post-content {  }
		'! !

!TinyBlogRootComponent methodsFor: 'updating' stamp: 'FrancoisJarriges 5/26/2016 13:49'!
updateRoot: anHtmlRoot
	"This method customize the <head> section of the root HTML document."
	"The API is very similar to the 'Canvas API' for rendering in the <body> section "
	"(where you 'draw' in #renderContentOn:)."
	"anHtmlRoot is an instance of WAHtmlRoot."

   super updateRoot: anHtmlRoot.
   anHtmlRoot beHtml5.
   anHtmlRoot title: TinyBlog current title! !

WAComponent subclass: #TinyBlogScreenComponent
	instanceVariableNames: 'header'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LostInMorvanBlog-Seaside'!

!TinyBlogScreenComponent commentStamp: 'FrancoisJarriges 5/28/2016 15:54' prior: 0!
I am an abstract class that defines  the common behaviour for  any component declared as main child of the TinyBlogRootComponent.

I declare to have at least a TinyBlogHeaderComponent  as child.
 
Internal Representation and Key Implementation Points.

    Instance Variables
	header: 		<TinyBlogHeaderComponent >


    Implementation Points!

TinyBlogScreenComponent subclass: #TinyBlogAdministrationComponent
	instanceVariableNames: 'report'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LostInMorvanBlog-Seaside'!

!TinyBlogAdministrationComponent commentStamp: 'FrancoisJarriges 5/29/2016 08:58' prior: 0!
I represent a variant of the main TinyBlogScreenComponent used for administration purpose (blog posts management).
I am using a Magritte automatic report built on the current blog and add it to my children list.

    Instance Variables
	report:		<MAReport>


    Implementation Points!

!TinyBlogAdministrationComponent methodsFor: 'hooks' stamp: 'FrancoisJarriges 5/29/2016 16:37'!
children
	"Add the Magritte report I build to my inherited children."
	
	^ super children copyWith: report
! !

!TinyBlogAdministrationComponent methodsFor: 'initialization' stamp: 'FrancoisJarriges 5/29/2016 11:53'!
initialize
	"Build a report based on the current blog"
   super initialize.
   report := TinyBlogPostsReport from: self blog! !

!TinyBlogAdministrationComponent methodsFor: 'rendering' stamp: 'FrancoisJarriges 5/29/2016 20:21'!
renderContentOn: html

	super renderContentOn: html.
	html tbsContainer: [ 
		html heading 
			level: 2;
			with: 'Blog Administration'.
		html anchor 	
			callback: [ self answer ];
			with: 'Public Area'.
		html horizontalRule.
		html render: report ]! !

!TinyBlogScreenComponent methodsFor: 'accessing' stamp: 'FrancoisJarriges 5/29/2016 20:44'!
blog
   "Return the current blog in the future we will ask the 
   session to return the blog of the currently logged user."

   ^ TinyBlog current! !

!TinyBlogScreenComponent methodsFor: 'hooks' stamp: 'FrancoisJarriges 5/26/2016 13:58'!
children
	"Seaside requires that each component knows and declares all its visible child components using the method WAComponent>>children. This allows Seaside to know in advance what components will be visible when building the HTML and configure and trigger some event on these components before the actual rendering happens."
	"Here we declare that our TinyBlogHeader instance variable is a child component of the underlying component"
	
   ^ OrderedCollection with: header! !

!TinyBlogScreenComponent methodsFor: 'initialization' stamp: 'FrancoisJarriges 5/26/2016 13:47'!
initialize
	"Create a TinyBlogHeaderComponent for this new component."

	super initialize.
	header := TinyBlogHeaderComponent new! !

!TinyBlogScreenComponent methodsFor: 'rendering' stamp: 'FrancoisJarriges 5/29/2016 07:53'!
renderContentOn: html
	"At this upper definition of the main component we make sure the common header is rendered first."
	
   html render: header! !

TinyBlogScreenComponent subclass: #TinyPostsListComponent
	instanceVariableNames: 'categoriesComponent postComponents'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LostInMorvanBlog-Seaside'!

!TinyPostsListComponent commentStamp: 'FrancoisJarriges 5/29/2016 08:30' prior: 0!
I represent a TinyBlogScreenComponent used to render a  list of  posts in this tiny blog application. 

I contain one TinyBlogCategoriesComponent child and a list of TinyPostComponent children, according to the category  selected in the TinyBlogCategoriesComponent.
 
Internal Representation and Key Implementation Points.

    Instance Variables
	categoriesComponent: 	<TInyBlogCategoriesComponent>
	postComponents: 		<OrderedCollection>


    Implementation Points!

!TinyPostsListComponent methodsFor: 'rendering' stamp: 'FrancoisJarriges 5/29/2016 07:50'!
buildRenderPostsOn: html

	postComponents removeAll.
	self readSelectedPosts do: [ :eachPost |
		
		| newPostComponent |
		
		newPostComponent := TinyPostComponent withPost: eachPost.
		postComponents add: newPostComponent.
		html render: newPostComponent ]! !

!TinyPostsListComponent methodsFor: 'hooks' stamp: 'FrancoisJarriges 5/29/2016 16:37'!
children
	"Add my sub-components to my inherited children."
	
	| children |
		
	children := super children.
	children add: categoriesComponent.
	postComponents do: [ :eachPostComponent |
		children add: eachPostComponent ].
	^ children! !

!TinyPostsListComponent methodsFor: 'initialization' stamp: 'FrancoisJarriges 5/30/2016 02:47'!
initialize
	"Create my sub-components and store them in the corresponding instance variables"
	
	super initialize.
	
	categoriesComponent := TinyBlogCategoriesComponent 
		categories: self blog allCategoriesFromVisibleBlogPosts.
		
	postComponents := OrderedCollection new.
	self blog allVisibleBlogPosts do: [ :eachPost |
		postComponents add: (TinyPostComponent withPost: eachPost) ]
	
! !

!TinyPostsListComponent methodsFor: 'data' stamp: 'FrancoisJarriges 5/29/2016 20:35'!
readSelectedPosts
	"Select the list of posts corresponding to the selected category"

	^ categoriesComponent selectedCategory 
		ifNil: [ self blog allVisibleBlogPosts ]
		ifNotNil: [ self blog allVisibleBlogPostsFromCategory: categoriesComponent selectedCategory ]
! !

!TinyPostsListComponent methodsFor: 'rendering' stamp: 'FrancoisJarriges 5/30/2016 02:46'!
renderCategoriesColumnOn: html

	categoriesComponent categories: self blog allCategoriesFromVisibleBlogPosts.
	html tbsColumn
		extraSmallSize: 12; smallSize: 3;	mediumSize: 4; largeSize: 3; largeOffset: 1;
		with: [ html render: categoriesComponent ]! !

!TinyPostsListComponent methodsFor: 'rendering' stamp: 'FrancoisJarriges 5/30/2016 09:19'!
renderContentOn: html
	"Create two sub-components : one for the categories list and the other for the posts list."
	"And get the rendering from them according to the category selected."
	"This method uses the Bootstrap Grid system in order to implement responsive rendering."
	
	super renderContentOn: html.
	self renderSignInOn: html.
	html render: (TinyBlogAuthenticationComponent new).
	html tbsContainer: [ 
		html tbsRow showGrid;
		with: [ 
			self renderCategoriesColumnOn: html.				
			self renderPostsColumnOn: html ] ]
	

	! !

!TinyPostsListComponent methodsFor: 'rendering' stamp: 'FrancoisJarriges 5/29/2016 07:50'!
renderPostsColumnOn: html

	html tbsColumn 
		extraSmallSize: 12; smallSize: 9; mediumSize: 8; largeSize: 8;
		with: [ self buildRenderPostsOn: html ]! !

!TinyPostsListComponent methodsFor: 'rendering' stamp: 'FrancoisJarriges 5/30/2016 08:50'!
renderSignInOn: html
	"Render a glyph with a link to either the authentication dialog or the administration component"
	
	html div 
		class: 'sign-in';
		with: [ 
			self session isLogged
				ifFalse: [
					html tbsGlyphIcon perform: #iconLock.
					html html: '<a data-toggle="modal" href="#TinyBlogAuthDialog" class="link"> Sign In</a>' ]
				ifTrue: [
					html tbsGlyphIcon perform: #iconUser.
					html anchor 
						callback: [ self call: TinyBlogAdministrationComponent new ]; 
						with: ' Private Area' ]]! !

WAComponent subclass: #TinyPostComponent
	instanceVariableNames: 'post'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LostInMorvanBlog-Seaside'!

!TinyPostComponent commentStamp: 'FrancoisJarriges 5/29/2016 08:51' prior: 0!
I represent a post content rendering  component for the TinyBlog application. I get the content of the post I am rendering from my parent component at creation time.
 
Internal Representation and Key Implementation Points.

    Instance Variables
	post:		<TinyPost>


    Implementation Points!

!TinyPostComponent class methodsFor: 'instance-creation' stamp: 'FrancoisJarriges 5/26/2016 16:06'!
withPost: aTinyPost

	^ self new post: aTinyPost
	
	! !

!TinyPostComponent methodsFor: 'accessing' stamp: 'FrancoisJarriges 5/26/2016 16:02'!
date

	^ post date! !

!TinyPostComponent methodsFor: 'private - accessing' stamp: 'FrancoisJarriges 5/26/2016 16:07'!
post: aTinyPost

	post := aTinyPost

	! !

!TinyPostComponent methodsFor: 'rendering' stamp: 'FrancoisJarriges 5/29/2016 06:50'!
renderContentOn: html

	html div 
		class: 'post-content';
		with: [ 
			html heading level: 2; with: self title.
   			html heading level: 6; with: self date.
   			html text: self text ]

   ! !

!TinyPostComponent methodsFor: 'accessing' stamp: 'FrancoisJarriges 5/26/2016 16:02'!
text

	^ post text! !

!TinyPostComponent methodsFor: 'accessing' stamp: 'FrancoisJarriges 5/26/2016 16:02'!
title

	^ post title! !

WASession subclass: #TinyBlogSession
	instanceVariableNames: 'logged'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LostInMorvanBlog-Seaside'!

!TinyBlogSession commentStamp: 'FrancoisJarriges 5/30/2016 08:55' prior: 0!
I represent session data  allocated to each TinyBlog application. 
 
Internal Representation and Key Implementation Points.

    Instance Variables
	logged:		<Boolean>


    Implementation Points!

!TinyBlogSession methodsFor: 'initialization' stamp: 'FrancoisJarriges 5/29/2016 19:58'!
initialize

	super initialize.
	logged := false! !

!TinyBlogSession methodsFor: 'accessing' stamp: 'FrancoisJarriges 5/29/2016 19:59'!
isLogged

	^ logged! !

!TinyBlogSession methodsFor: 'accessing' stamp: 'FrancoisJarriges 5/29/2016 20:17'!
logged: aBoolean
	logged := aBoolean! !

Object subclass: #TinyBlog
	instanceVariableNames: 'title posts'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LostInMorvanBlog'!

!TinyBlog commentStamp: 'FrancoisJarriges 5/26/2016 13:29' prior: 0!
I represent a TinyBlog kind of blog (FUN MOOC - Programmation immersive en Pharo)

I provide methods on  the class side to manage a  unique instance of myself 

I provide methods to add and remove posts to myself and manage them through the criterias defined by  their class TinyPost

Public API and Key Messages :

aPost := TinyPost title: 'A title'  text: 'A text'  category: 'A category' .

blog :=  TinyBlog current.		"unique instance of TinyBlog"
blog addBlogPost:  aPost.		"add a post to the blog"
blog removeBlogPost:  aPost. 	"remove a post from the blog"

"Get selected posts in an OrderedCollection of posts :"

allPosts :=  blog allBlogPosts.
postsFromACategory :=	blog allBlogPostsFromCategory:  'A category'.
visiblePosts := blog allVisibleBlogPosts.
notVisiblePosts := blog allVisibleBlogPosts.
visiblePostsFromACategory := blog allVisibleBlogPostsFromCategory:  'A category'.
notVisiblePostsFromACategory := blog allNotVisibleBlogPostsFromCategory: 'A category'.

"Get  an actual Set  of   categories among the selected posts :"

allCategories := blog allCategories.
categoriesFromVisiblePosts := blog allCategoriesFromVisibleBlogPosts.
categoriesFromNotVisiblePosts := blog allCategoriesFromNotVisibleBlogPosts.

"Get the actual number of posts :"

numberOfPosts := blog size.

"Remove a selection of posts from the blog :"

blog removeAllBlogPostsFromCategory: 'A category'.
blog removeAllPosts .

Internal Representation and Key Implementation Points.

    Instance Variables
	title: 	<ByteString>
	posts:	<OrderedCollection>
		
    Implementation Points
	A TinyBlog must be  instanciated through the method #current in order to be managed as a singleton
	A #createDemoPosts method is defined on the class side of TinyBlog to allow the creation of a test blog by TinyBlogTest
	
	
	
	!

!TinyBlog class methodsFor: 'tests' stamp: 'FrancoisJarriges 5/30/2016 02:24'!
createDemoPosts
"Create a demo posts for testing the TinyBlog application"
	
(self current allBlogPosts size > 0) ifTrue: [ self current removeAllBlogPosts ].

self current
	
	title: 'demoPosts';

	addBlogPost: ((TinyPost 
		title: 'Welcome in TinyBlog' 
		text: 'TinyBlog is a small blog engine made with Pharo.' 
		category: 'TinyBlog') 
		visible: true);
		
	addBlogPost: ((TinyPost 
		title: 'Report Pharo Sprint' 
		text: 'Friday, June 12 there was a Pharo sprint / Moose dojo. It was a nice event with more than 15 motivated sprinters. With the help of candies, cakes and chocolate, huge work has been done'
		category: 'Pharo') 
		visible: true);
		
	addBlogPost: ((TinyPost 
		title: 'Brick on top of Bloc - Preview' 
		text: 'We are happy to announce the first preview version of Brick, a new widget set created from scratch on top of Bloc. Brick is being developed primarily by Alex Syrel (together with Alain Plantec, Andrei Chis and myself), and the work is sponsored by ESUG. Brick is part of the Glamorous Toolkit effort and will provide the basis for the new versions of the development tools.'
		category: 'Pharo') 
		visible: true);
		
	addBlogPost: ((TinyPost 
		title: 'The sad story of unclassified blog posts'
		text: 'So sad that I can read this.') 
		visible: true);
		
	addBlogPost: ((TinyPost 
		title: 'Working with Pharo on the Raspberry Pi' 
		text: 'Hardware is getting cheaper and many new small devices like the famous Raspberry Pi provide new computation power that was one once only available on regular desktop computers.' 
		category: 'Pharo') 
		visible: true)! !

!TinyBlog class methodsFor: 'accessing' stamp: 'FrancoisJarriges 5/26/2016 14:55'!
current
	"Answer the unique instance of the TinyBlog and create it if it doesn't exist yet."
	
		^ self selectAll 
			ifNotEmpty: [ :x | x anyOne ]
			ifEmpty: [ self new save ]! !

!TinyBlog class methodsFor: 'class initialization' stamp: 'FrancoisJarriges 5/26/2016 14:55'!
initialize
	"The data base is reinitialized each time the code is loaded."

	self reset ! !

TinyBlog initialize!

!TinyBlog class methodsFor: 'voyage' stamp: 'FrancoisJarriges 5/26/2016 11:06'!
initializeVoyageInMemory
	"Create, initialize and connect to a data base stored in memory"
	"The enableSingleton method indicates to Voyage that will use only one database"
	"(thus we don't have to designate it)"
	
	| repository |
	
	repository := VOMemoryRepository new.
	repository enableSingleton
! !

!TinyBlog class methodsFor: 'voyage' stamp: 'FrancoisJarriges 5/26/2016 14:54'!
initializeVoyageOnLocalMongoDb
	"Initialize and connect to a data base stored on a local server"
	"The enableSingleton method indicates to Voyage that will use only one database"
	"(thus we don't have to designate it)"
	
	| repository |
	
	VORepository setRepository: nil.
	repository := VOMongoRepository database: TinyBlog current title.
	repository enableSingleton! !

!TinyBlog class methodsFor: 'voyage' stamp: 'FrancoisJarriges 5/26/2016 14:52'!
isVoyageRoot
	"Indicates that instances of this class are top level document in noSQL databases"
	
	^ true
! !

!TinyBlog class methodsFor: 'class initialization' stamp: 'FrancoisJarriges 5/26/2016 14:55'!
reset
	"Reinitialize the current Voyage data base."

	self initializeVoyageInMemory
	"self initializeVoyageOnLocalMongoDb"! !

!TinyBlog methodsFor: 'adding/removing' stamp: 'FrancoisJarriges 5/26/2016 14:59'!
addBlogPost: aTinyPost
	"Add the given post to the current blog database."
	"If needed save the new state of the TinyBlog instance to the Voyage repository."
	
	posts add: aTinyPost.
	self save ! !

!TinyBlog methodsFor: 'actions' stamp: 'FrancoisJarriges 5/26/2016 14:59'!
allBlogPosts
	"Answer the entire posts collection defined in the current blog."

	^ posts! !

!TinyBlog methodsFor: 'actions' stamp: 'FrancoisJarriges 5/26/2016 14:59'!
allBlogPostsFromCategory: aString 
	"Answer a selection of all the posts classified in the given category from the current blog."
	
	^ posts select: [ :eachPost | eachPost category = aString ]! !

!TinyBlog methodsFor: 'actions' stamp: 'FrancoisJarriges 5/26/2016 15:00'!
allCategories
	"Answer a set of all the posts categories defined in the current blog."
	
	^ (posts collect: [ :eachPost | eachPost category ]) asSet! !

!TinyBlog methodsFor: 'actions' stamp: 'FrancoisJarriges 5/27/2016 11:31'!
allCategoriesFromNotVisibleBlogPosts
	"Answer a set of all the categories defined among unvisible posts from the current blog."

	^ ((posts reject: [ :eachPost | eachPost isVisible ])
		collect: [ :eachNotVisiblePost | eachNotVisiblePost category ])
		asSet! !

!TinyBlog methodsFor: 'actions' stamp: 'FrancoisJarriges 5/26/2016 15:00'!
allCategoriesFromVisibleBlogPosts
	"Answer a set of all the categories defined among visible posts from the current blog."
	
	^ ((posts select: [ :eachPost | eachPost isVisible ]) 
		collect: [ :eachVisiblePost | eachVisiblePost category ]) asSet! !

!TinyBlog methodsFor: 'actions' stamp: 'FrancoisJarriges 5/27/2016 11:31'!
allNotVisibleBlogPosts
	"Answer a selection of all the unvisible posts from the current blog."

	^ posts reject: [ :eachPost | eachPost isVisible ]! !

!TinyBlog methodsFor: 'actions' stamp: 'FrancoisJarriges 5/26/2016 15:00'!
allNotVisibleBlogPostsFromCategory: aString 
	"Answer a selection of all the unvisible posts in the given category from the current blog."
	
	^ posts select: [ :eachPost | (eachPost category = aString) & (eachPost isVisible not) ]! !

!TinyBlog methodsFor: 'actions' stamp: 'FrancoisJarriges 5/26/2016 15:00'!
allVisibleBlogPosts
	"Answer a selection of all the visible posts from the current blog."
	
	^ posts select: [ :eachPost | eachPost isVisible ]! !

!TinyBlog methodsFor: 'actions' stamp: 'FrancoisJarriges 5/26/2016 15:00'!
allVisibleBlogPostsFromCategory: aString 
	"Answer a selection of all the visible posts in the given category from the current blog."
	
	^ posts select: [ :eachPost | (eachPost category = aString) & (eachPost isVisible) ]! !

!TinyBlog methodsFor: 'actions' stamp: 'FrancoisJarriges 5/26/2016 15:01'!
categoryBeNotVisible: aString 
	"Make each post unvisible within the given category identified as aString."
	"Answer a selection of all the posts classified in the given category from the current blog."
	"If needed save the new state of the TinyBlog instance to the Voyage repository."
	
	| postsFromSelectedCategory |
	
	postsFromSelectedCategory := self allBlogPostsFromCategory: aString.
	postsFromSelectedCategory do: [ :eachPostFromSelectedCategory | eachPostFromSelectedCategory beNotVisible ].
	self save.
	^ postsFromSelectedCategory! !

!TinyBlog methodsFor: 'actions' stamp: 'FrancoisJarriges 5/26/2016 15:01'!
categoryBeVisible: aString 
	"Make each post visible within the given category identified as aString."
	"Answer a selection of all the posts classified in the given category from the current blog."
	"If needed save the new state of the TinyBlog instance to the Voyage repository."
	
	| postsFromSelectedCategory |
	
	postsFromSelectedCategory := self allBlogPostsFromCategory: aString.
	postsFromSelectedCategory do: [ :eachPostFromSelectedCategory | eachPostFromSelectedCategory beVisible ].
	self save.
	^ postsFromSelectedCategory! !

!TinyBlog methodsFor: 'actions' stamp: 'FrancoisJarriges 5/26/2016 15:01'!
categoryRenameFrom: oldString to: newString
	"Rename the actual category identified as actualString by giving it newString as new name."
	"Answer a selection of all the posts classified in the given category from the current blog."
	"If needed save the new state of the TinyBlog instance to the Voyage repository."

	| postsFromSelectedCategory |
	
	postsFromSelectedCategory := self allBlogPostsFromCategory: oldString.
	postsFromSelectedCategory do: [ :eachPostFromSelectedCategory | 
		eachPostFromSelectedCategory category: newString ].
	self save.
	^ postsFromSelectedCategory! !

!TinyBlog methodsFor: 'initialization' stamp: 'FrancoisJarriges 5/26/2016 15:01'!
initialize

	super initialize.
	posts := OrderedCollection new! !

!TinyBlog methodsFor: 'adding/removing' stamp: 'FrancoisJarriges 5/30/2016 02:17'!
removeAllBlogPosts
	"Remove all posts by creating a new empty collection for the current blog."
	"If needed save the new state of the TinyBlog instance to the Voyage repository."

	posts := OrderedCollection new.
	self save! !

!TinyBlog methodsFor: 'adding/removing' stamp: 'FrancoisJarriges 5/30/2016 02:18'!
removeAllBlogPostsFromCategory: aString
	"Remove all the posts classified in the given category from the current blog."
	"If needed save the new state of the TinyBlog instance to the Voyage repository."

	posts	removeAllSuchThat: [ :eachPost | eachPost category = aString ].
	self save! !

!TinyBlog methodsFor: 'adding/removing' stamp: 'FrancoisJarriges 5/30/2016 02:18'!
removeBlogPost: aTinyPost 
	"Remove the given post from the current blog database."
	"If needed save the new state of the TinyBlog instance to the Voyage repository."

	posts remove: aTinyPost.
	self save! !

!TinyBlog methodsFor: 'accessing' stamp: 'FrancoisJarriges 5/26/2016 14:58'!
size
	"Answer the size of the posts collection in the current blog."

	^ posts size! !

!TinyBlog methodsFor: 'accessing' stamp: 'FrancoisJarriges 5/26/2016 14:58'!
title

	^ title ifNil: [ title := 'no title' ]! !

!TinyBlog methodsFor: 'accessing' stamp: 'FrancoisJarriges 5/26/2016 14:56'!
title: aByteString

	title := aByteString! !

Object subclass: #TinyPost
	instanceVariableNames: 'title text date category visible'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LostInMorvanBlog'!

!TinyPost commentStamp: 'FrancoisJarriges 5/10/2016 14:09' prior: 0!
I represent a post in the Tiny Blog (FUN MOOC - Programmation immersive en Pharo)

I provide methods to create and manage  a post


Public API and Key Messages :

post1 := TBMPost title: 'test post 1'	text: 'created unclassified and unvisible' .
post2 := TBMPost title: 'test post 2'  text: 'created unvisible'  category: 'Testing' .

post1 isUnclassified ifTrue:  [post1 category: 'Testing'] .
post1 isVisible  ifFalse:  [post1 beVisible ] .

post1 beNotVisible; category: self class unclassifiedTag .



Internal Representation and Key Implementation Points.

    Instance Variables
	category:	<String>
	date:		<Date>
	text:		<Text>
	title:		<String>
	visible:		<Boolean>


    Implementation Points!

!TinyPost class methodsFor: 'instance-creation' stamp: 'FrancoisJarriges 5/5/2016 10:47'!
title: aTitle text: aText

	^ self new 
		title: aTitle;
		text: aText;
		yourself ! !

!TinyPost class methodsFor: 'instance-creation' stamp: 'FrancoisJarriges 5/5/2016 10:49'!
title: aTitle text: aText category: aCategory

	^ (self title: aTitle text: aText) 
		category: aCategory;
		yourself ! !

!TinyPost class methodsFor: 'accessing' stamp: 'FrancoisJarriges 5/4/2016 17:08'!
unclassifiedTag

	^ 'Unclassified'! !

!TinyPost methodsFor: 'actions' stamp: 'FrancoisJarriges 5/4/2016 17:04'!
beNotVisible

	self visible: false! !

!TinyPost methodsFor: 'actions' stamp: 'FrancoisJarriges 5/4/2016 17:03'!
beVisible

	self visible: true! !

!TinyPost methodsFor: 'accessing' stamp: 'FrancoisJarriges 5/4/2016 17:01'!
category

	^ category! !

!TinyPost methodsFor: 'accessing' stamp: 'FrancoisJarriges 5/9/2016 08:36'!
category: aString

	category := aString! !

!TinyPost methodsFor: 'accessing' stamp: 'FrancoisJarriges 5/4/2016 17:00'!
date

	^ date! !

!TinyPost methodsFor: 'accessing' stamp: 'FrancoisJarriges 5/9/2016 08:54'!
date: aDate

	date := aDate! !

!TinyPost methodsFor: 'descriptions' stamp: 'FrancoisJarriges 5/30/2016 01:55'!
descriptionCategory
	<magritteDescription>
	^ MAStringDescription new
		label: 'Category';
		priority: 300;
		accessor: #category;
		fromStringCollection: TinyBlog current allCategories;
		comment: 'Unclassified if empty';
		componentClass: TBSMagritteTextInputComponent;
		yourself! !

!TinyPost methodsFor: 'descriptions' stamp: 'FrancoisJarriges 5/28/2016 14:39'!
descriptionContainer
	"The ContainerDescription class and its subclasses provide a collection container for other descriptions. In fact the container implements the whole collection interface, so that users can easily iterate (do:), filter (select:, reject:), transform (collect:) and query (detect:, anySatisfy:, allSatisfy:) the containing descriptions.
	The componentRenderer method is served by MAContainer class to specify the properties of the container."
	
	<magritteContainer>
	^ super descriptionContainer
		componentRenderer: TBSMagritteFormRenderer;
		yourself! !

!TinyPost methodsFor: 'descriptions' stamp: 'FrancoisJarriges 5/30/2016 01:54'!
descriptionDate
   <magritteDescription>
   ^ MADateDescription new
      label: 'Date';
      priority: 400;
      accessor: #date;
      beRequired;
		addCondition: [ :value | value >= Date today ];
      yourself! !

!TinyPost methodsFor: 'descriptions' stamp: 'FrancoisJarriges 5/28/2016 12:40'!
descriptionText
	<magritteDescription>
	^ MAMemoDescription new
		label: 'Text';
		priority: 200;
		accessor: #text;
		beRequired;
		requiredErrorMessage: 'A blog post must contain a text.';
		comment: 'Please enter a text';
		componentClass: TBSMagritteTextAreaComponent;
		yourself! !

!TinyPost methodsFor: 'descriptions' stamp: 'FrancoisJarriges 5/28/2016 12:40'!
descriptionTitle
	<magritteDescription>
	^ MAStringDescription new
		label: 'Title';
		priority: 100;
		accessor: #title;
		requiredErrorMessage: 'A blog post must have a title.';
		comment: 'Please enter a title';
		componentClass: TBSMagritteTextInputComponent;
		beRequired;
		yourself! !

!TinyPost methodsFor: 'descriptions' stamp: 'FrancoisJarriges 5/28/2016 12:41'!
descriptionVisible
	<magritteDescription>
	^ MABooleanDescription new
		checkboxLabel: 'Visible';
		priority: 500;
		accessor: #visible;
		componentClass: TBSMagritteCheckboxComponent;
		beRequired;
		yourself! !

!TinyPost methodsFor: 'initialization' stamp: 'FrancoisJarriges 5/10/2016 14:11'!
initialize

	super initialize.
	self category: self class unclassifiedTag.
	self date: Date today.
	self beNotVisible ! !

!TinyPost methodsFor: 'testing' stamp: 'FrancoisJarriges 5/5/2016 10:57'!
isUnclassified

	^ self category = self class unclassifiedTag ! !

!TinyPost methodsFor: 'testing' stamp: 'FrancoisJarriges 5/4/2016 17:04'!
isVisible

	^ self visible! !

!TinyPost methodsFor: 'accessing' stamp: 'FrancoisJarriges 5/4/2016 16:59'!
text

	^ text! !

!TinyPost methodsFor: 'accessing' stamp: 'FrancoisJarriges 5/9/2016 08:20'!
text: aText

	text := aText ! !

!TinyPost methodsFor: 'accessing' stamp: 'FrancoisJarriges 5/4/2016 16:58'!
title

	^ title! !

!TinyPost methodsFor: 'accessing' stamp: 'FrancoisJarriges 5/9/2016 08:31'!
title: aString

	title := aString ! !

!TinyPost methodsFor: 'accessing' stamp: 'FrancoisJarriges 5/4/2016 17:01'!
visible

	^ visible! !

!TinyPost methodsFor: 'accessing' stamp: 'FrancoisJarriges 5/5/2016 10:34'!
visible: aBoolean

	visible := aBoolean  ! !

Object subclass: #TinyTeapotWebApp
	instanceVariableNames: 'teapot'
	classVariableNames: 'Server'
	poolDictionaries: ''
	category: 'LostInMorvanBlog-Teapot'!

!TinyTeapotWebApp commentStamp: 'FrancoisJarriges 5/22/2016 14:45' prior: 0!
I provide a web application interface for the TinyBlog by using the Teapot framework  (http://smalltalkhub.com/#!!/~zeroflag/Teapot)

Messages to start and stop this web application:

TinyTeapotWebApp start.
TinyTeapotWebApp stop.

Once started, the application can be accessed through the URI http://localhost:8081/ by a web browser.
 
Internal Representation and Key Implementation Points.

    Instance Variables
	teapot	<Teapot>

    Class Variables 
	Server 	<TinyTeapotWebApp>

    Implementation Points!

!TinyTeapotWebApp class methodsFor: 'accessing' stamp: 'FrancoisJarriges 5/19/2016 12:35'!
server

	^ Server! !

!TinyTeapotWebApp class methodsFor: 'starting/stopping' stamp: 'FrancoisJarriges 5/18/2016 10:19'!
start

	Server ifNil: [ Server := self new start ]! !

!TinyTeapotWebApp class methodsFor: 'starting/stopping' stamp: 'FrancoisJarriges 5/18/2016 10:20'!
stop

	Server ifNotNil: [ Server stop. Server := nil ]! !

!TinyTeapotWebApp methodsFor: 'accessing' stamp: 'FrancoisJarriges 5/22/2016 13:44'!
allCategories

	 ^ TinyBlog current allCategoriesFromVisibleBlogPosts ! !

!TinyTeapotWebApp methodsFor: 'accessing' stamp: 'FrancoisJarriges 5/22/2016 13:44'!
allPosts

	 ^ TinyBlog current allVisibleBlogPosts ! !

!TinyTeapotWebApp methodsFor: 'accessing' stamp: 'FrancoisJarriges 5/22/2016 13:44'!
allPostsForCategory: aCategory

	 ^ TinyBlog current allVisibleBlogPostsFromCategory: aCategory  ! !

!TinyTeapotWebApp methodsFor: 'html' stamp: 'FrancoisJarriges 5/27/2016 12:33'!
allPostsPage
	^ String
		streamContents: [ :newStream | 
			self renderPageHeaderOn: newStream.
			newStream 
				<< '<h1><a class="root" href="/">TinyBlog</a></h1>';
				
				<< '<p class="category">All categories</p>';
				
				<< '<ul>'.
			self allPosts
				keysAndValuesDo: [ :index :aPost | 
					newStream
						<< '<li><a href="/all/';
						<< index asString; 
						<< '">';
						<< aPost title;
						<< '</a></li>' ].
			newStream << '</ul>'.
			
			self renderPageFooterOn: newStream ]! !

!TinyTeapotWebApp methodsFor: 'html' stamp: 'FrancoisJarriges 5/18/2016 20:01'!
errorPage

	^ String streamContents: [ :newStream |
		self renderPageHeaderOn: newStream.
		newStream << '<p>Oups, an error occured</p>'.
		self renderPageFooterOn: newStream ]
! !

!TinyTeapotWebApp methodsFor: 'html' stamp: 'FrancoisJarriges 5/27/2016 12:34'!
homePage

	^ String streamContents: [ :newStream |
		self renderPageHeaderOn: newStream.
		newStream 
			<< '<h1>TinyBlog</h1>';
			
			<< '<p class="category">Categories</p>';
			
			<< '<ul><h3>'.
		self allCategories do: [ :aCategory |
			newStream << '<li><a href="/';
				<< aCategory;
				<< '">';
				<< aCategory;
				<< '</a></li>' ].
		newStream 
			<< '<li><a href="/all">All</a></li>';
			<< '</h3></ul>'.
			
		self renderPageFooterOn: newStream ]
! !

!TinyTeapotWebApp methodsFor: 'initialization' stamp: 'FrancoisJarriges 5/27/2016 12:08'!
initialize
	
	super initialize.
	teapot := Teapot configure: { 
		#port -> 8081.
		#debugMode -> true }! !

!TinyTeapotWebApp methodsFor: 'html' stamp: 'FrancoisJarriges 5/27/2016 12:34'!
pageForCategoryName: aCategory
	(self allCategories includes: aCategory)
		ifFalse: [ ^ self errorPage ].
	^ String
		streamContents: [ :newStream | 
			self renderPageHeaderOn: newStream.
			newStream 
				<< '<h1><a class="root" href="/">TinyBlog</a></h1>';
				
				<< '<p class="category">Category: ';
				<< aCategory; 
				<< '</p>';
				
				<< '<ul>'.
			(self allPostsForCategory: aCategory)
				keysAndValuesDo: [ :index :aPost | 
					newStream
						<< '<li><a href="/'; 
						<< aCategory; 
						<< '/';
						<< index asString;
						<< '">';
						<< aPost title;
						<< '</a></li>' ].
			newStream << '</ul>'.
			
			self renderPageFooterOn: newStream ]! !

!TinyTeapotWebApp methodsFor: 'html' stamp: 'FrancoisJarriges 5/27/2016 12:35'!
pageForPostNumber: aPostNumber

	| currentPost |
	
	(aPostNumber between: 1 and: self allPosts size) ifFalse: [ ^ self errorPage ].
	currentPost := self allPosts at: aPostNumber.
	^ String streamContents: [ :newStream |
		self renderPageHeaderOn: newStream.
		newStream 
			<< '<h1><a class="root" href="/">TinyBlog</a></h1>';
			
			<< '<p class="category"><a href="/all">All categories</a></p>';
			
			<< '<p class="date">';
			<< currentPost date mmddyyyy;
			<< '</p>';
			
			<< '<h2>';
			<< currentPost title;
			<< '</h2>';
			
			<< '<p>';
			<< currentPost text;
			<< '</p>'.
			
		self renderPageFooterOn: newStream ]! !

!TinyTeapotWebApp methodsFor: 'html' stamp: 'FrancoisJarriges 5/27/2016 12:35'!
pageForPostNumber: aPostNumber inCategory: aCategory

	| currentPost |
	
	(aPostNumber between: 1 and: (self allPostsForCategory: aCategory) size) ifFalse: [ ^ self errorPage ].
	currentPost := (self allPostsForCategory: aCategory) at: aPostNumber.
	^ String streamContents: [ :newStream |
		self renderPageHeaderOn: newStream.
		newStream 
			<< '<h1><a class="root" href="/">TinyBlog</a></h1>';
			
			<< '<p class="category"> Category: <a href="/';
			<< aCategory;
			<< '">';
			<< currentPost category;
			<< '</a></p>';
			
			<< '<p class="date">';
			<< currentPost date mmddyyyy;
			<< '</p>';
			
			<< '<h2>';
			<< currentPost title;
			<< '</h2>';
			
		   << '<p>';
			<< currentPost text;
			<< '</p>'.
			
		self renderPageFooterOn: newStream ]! !

!TinyTeapotWebApp methodsFor: 'html' stamp: 'FrancoisJarriges 5/18/2016 11:10'!
renderPageFooterOn: aStream 

	aStream << '</body></html>'! !

!TinyTeapotWebApp methodsFor: 'html' stamp: 'FrancoisJarriges 5/19/2016 15:07'!
renderPageHeaderOn: aStream 
	aStream << '
		<!!DOCTYPE html>
		<html>
		<head>
		<style>
			body { 
			   font-family: Arial, Helvetica, sans-serif;
				color: white;
				background-color: #181818; 
			}
			h1 { 
				font-family: "Arial Black", Gadget, sans-serif;
			}
			a { 
				color: orange;
				text-decoration: none;
			}
			a.root { 
				color: white;
			}
			p {
    			border: 1px solid white;
    			padding: 30px;
			}
			p.date { 
				border: 0;
				padding-left: 0;
				padding-bottom: 0;
			}
			p.category { 
				border: 0;
				padding: 10px;
				background-color: black;
			}
		</style>
		</head>
		<body>'! !

!TinyTeapotWebApp methodsFor: 'starting/stopping' stamp: 'FrancoisJarriges 5/19/2016 12:34'!
start
	
	teapot
		GET: '/' -> [ self homePage ];
		GET: '/all' -> [ self allPostsPage ];
		GET: '/all/<id>' -> [ :request |
			self pageForPostNumber: (request at: #id) asNumber ];
		GET: '/<name>' -> [ :request |
			self pageForCategoryName: (request at: #name) ];
		GET: '/<name>/<id>' -> [  :request |
			self pageForPostNumber: (request at: #id) asNumber inCategory: (request at: #name) ];
		start! !

!TinyTeapotWebApp methodsFor: 'starting/stopping' stamp: 'FrancoisJarriges 5/18/2016 10:15'!
stop
	
	teapot stop
! !

TinyBlog initialize!

TinyBlogRootComponent initialize!
